//
//  Fractal.h
//  IFS3D
//
//  Copyright (c) 2001 David Leppik.
//

#import <Foundation/Foundation.h>

#import "IFS3DDefaults.h"
#import "MathFunctions.h"
#import <OpenGL/gl.h>
#import <OpenGL/glu.h>
#import <Cocoa/Cocoa.h>

#define LARGE_POINT_SIZE	3

enum ViewerEye { NEITHER, LEFT, RIGHT };


#define FRACTAL_CALL_LIST 10
#define FRACTAL_INTERNAL_CALL_LIST 11

// This defines the fractal structure in terms of affine transformations
struct branch {
    VECTOR translation;
    VECTOR rotation;
    VECTOR scale;
};


/** The fractal itself. */
@interface Fractal: NSObject {
    id	lastView;


    BOOL		isFlat;
    int 		iterations;
    int		branchCount; 
    float           speed;

    struct branch	branches[MAX_BRANCHES];  // fractal structure
    struct branch	velocity[MAX_BRANCHES];  // change in structure over time

    struct branch	maxBranch; // highest values allowed for each dimension of structure
    struct branch	minBranch; // lowest values allowed for each dimension of structure
    struct branch	maxDeltaV; // maximum change in velocity, related to speed
    float 		boundaryElasticity; // -1 to 0; change in velocity when limit is hit


    /** Generate the fractal using points. */
    enum ViewerEye eye;
    BOOL isStereo;

}

//------------------------------------------------------------------------------------
//  Update
//------------------------------------------------------------------------------------

- (void)updateBranches;

void walkFractal(int depth, MATRIX *m, MATRIX transform, int branchCount, Fractal *theShape);



/** If YES, this fractal has a left-eye and right-eye mode */
- (BOOL)canDoStereo;

/** If YES, the "Large Points" checkbox has some effect */
- (BOOL)usesLargePoints;


//------------------------------------------------------------------------------------
//  Drawing
//------------------------------------------------------------------------------------
/** If YES, this fractal is in stereo mode */
- (BOOL)isStereo;

- (void)draw3DWithView:(id) view;
- (void)setIsStereo:(bool)newIsStereo;

//------------------------------------------------------------------------------------
//  Getters and setters
//------------------------------------------------------------------------------------
    /** IFS3D_LEFT or IFS3D_RIGHT */
- (void)setEye:(enum ViewerEye)newEye;
- (enum ViewerEye)getEye;

/** If true, fractal is limited to 2 dimensions */
- (BOOL)isFlat;
- (void)setIsFlat:(BOOL)newIsFlat;

    /** Call callGLParams and set view-specific GL settings.
    *  This should be called before calling beginDrawing.
    */
- (void)setView:(id)view;
/** Recursive depth of the fractal */
- (int)iterations;
- (void)resetIterations;
- (void)increaseIterations;
- (void)decreaseIterations;

    /** Enable or disable render settings */
- (void)callGlParams;

/** Number of branches, a.k.a. complexity */
- (int)branchCount;
- (void)setBranchCount:(int) newBranchCount;
- (void)buildListWithBranchTransforms:(MATRIX *)m transform:(MATRIX) transform;

/** speed of updates */
- (float)speed;
- (void)setSpeed:(float) newSpeed;

    /** Draw the shape in the current OpenGL environment.
    *   Transformation points to 16 values which form a
    *   4x4 matrix.  Calling glMultMatrixf(transformation)
    *   will translate, rotate, and scale the primative
    *   into place, but this and all GL matrix operations
    *   should be avoided if it allows you to keep
    *   glBegin and glEnd in beginDrawing and endDrawing.  
    */
- (void)drawPrimitive:(GLfloat *) transformation;

@end
